; Hotkey switch to control LEDPANEL 3.0 display
; August 1995 by Mark Slagell

; This should be placed in the AUTO folder, if only because
; it would otherwise be susceptible to install on top of itself.

; tab width: 13

 .text
 
tsr_start:
	bra	tsr_end
	

; Here's the resident part ...

  ; CONSTANTS: Hotkey definition
c_shift:	dc.b  3	; both shift keys
c_scan:	dc.b  57	; scancode of spacebar
ikbd1:  ; The main idea here is to manipulate return addresses to
        ; retake control after the key interrupt handler.  First
        ; store the original return address for when interrupt
        ; processing is done.
	move.l	(sp),retikbd
        ; Get the keyboard buffer's tail index for later comparison.
        ; Use (sp) for temp storage -- we'll overwrite it soon anyway.
	move.l	a0,(sp)
	move.l	tailp(pc),a0
	move	(a0),o_tail
	move.l	(sp),a0
        ; Impose stage 2 start address so that code will be executed
        ; after the existing interrupt handler runs.
	move.l	#ikbd2,(sp)
        ; The following jump instruction will be modified during
        ; installation so it points to the existing interrupt handler.
ADJ1:	jmp	ADJ1

ikbd2:  ; The interrupt handler has done its thing, and now we
        ; must figure out if a key was pressed, and if so, which one.
	move	sr,sr_0
	or	#$700,sr
	movem.l	d0-d6/a0-a3,reg_0
        ; Get the new tail index, compare it to before.  If it didn't
        ; change, the interrupt wasn't caused by a new keystroke.
	move.l	tailp(pc),a1
	move	(a1),d0
	cmp	o_tail(pc),d0
	beq	i2_out
        ; Now that we know a key was pressed, we get its scancode out
        ; of the buffer, and see if it corresponds to the hotkey.
	ext.l	d0
	add.l	bufp(pc),d0
	move.l	d0,a0
	move.b	1(a0),d0
	cmp.b	c_scan(pc),d0
	bne	i2_out
        ; The correct key has been pressed.  Make sure the shift
        ; pattern also matches.
	move.l	shiftp(pc),a0
	move.b	(a0),d0
	and.b	#15,d0	; mask out capslock etc.
	cmp.b	c_shift(pc),d0
	bne	i2_out
        ; Okay, it's the hotkey.  Rub it out of the buffer and
        ; throw the LEDPANEL switch.
	move	o_tail(pc),(a1)
	tst.l	cookiep	; do we know where the cookie is?
	bne	throw	;  yes: throw the switch
	bsr	find_panel	; look for LEDP cookie
	tst.l	cookiep	; did we find it?
	beq	i2_out	;  no: abort
throw:	move.l	cookiep(pc),a3
	tst	(a3)
	bne	turn_off
turn_on:	move	#1,(a3)	; switch: on
	bra	i2_out
turn_off:    move	#0,(a3)	; switch: off
        ; Clear the rectangle occupied by LEDPANEL display.
	move.l	alinep(pc),a0
	lea	32(a3),a1	; GRECT pointer
	lea	v_x1(pc),a2
	move	(a1)+,d0
	move	d0,(a2)+	; v_x1 = g_x
	move	(a1)+,d1	; g_y, to be used in a moment
	add	(a1)+,d0	; g_x + g_w ...
	subq	#1,d0
	move	d0,(a2)	; v_x2 = g_x + g_w - 1
	move	(a1),d4	; scanlines to clear = g_h
	subq	#1,d4	; adjust for dbra
	move.l	$44e,a1	; _v_bas_ad (physbase)
	mulu	-2(a0),d1	; bytes to offset vertically
	add.l	d1,a1	; start address for clear
	move	-12(a0),d3	; d3 = horiz resolution (limit)
	moveq	#0,d0	; d0 is clear data (0 constant)
	moveq	#16,d5	; step value for d1 and d2
tflp4:	moveq	#0,d1	; left edge pixel counter
	moveq	#15,d2	; right edge pixel counter
tflp3:	cmp	v_x1(pc),d2
	blo	tfskip	; too far left: skip
	cmp	v_x2(pc),d1
	bhi	tfskip	; too far right: skip
tfclr:	move	(a0),d6	; number of bitplanes
	subq	#1,d6	; adjust for dbra
tflp6:	move	d0,(a1)+	; clear a word
	dbra	d6,tflp6	;  loop back
	bra	tfcomn
tfskip:	add	(a0),a1	; add # of bitplanes times two
	add	(a0),a1
tfcomn:	add	d5,d1	; add 16 to left edge counter,
	add	d5,d2	;  also to right edge counter
	cmp	d3,d1	; did left edge fall off screen?
	blo	tflp3	;  no: loop back
	dbra	d4,tflp4	;  yes: do next scan line
	
i2_out:	movem.l	reg_0,d0-d6/a0-a3
	move	sr_0,sr
        ; The following is equiv to "move.l retikbd(pc),a0"/"jmp (a0)",
        ; except it doesn't pollute a register.
	move.l	retikbd(pc),-(sp)
	rts
find_panel:	
	move.l	$5a0,d0
	beq	fp_out	; no cookie jar, so punt
	move.l	d0,a0
fp_loop:	move.l	(a0)+,d0	; get next cookie, ID longword
	beq	fp_out	; end of jar, so punt
	move.l	(a0)+,d1	; get cookie pointer longword
	cmp.l	#$4c454450,d0 ; LEDP cookie?
	bne	fp_loop	;  no: loop back
	add.l	#14,d1	;  yes: point to switch word
	move.l	d1,cookiep
fp_out:	rts


  ; resident variables (these must stay in the text segment)
tailp:	dc.l	0	; pointer to tail index
bufp:	dc.l	0	; pointer to key buffer
alinep:	dc.l	0	; pointer to a-line var table
cookiep:	dc.l	0	; pointer to 'active' switch
shiftp:	dc.l	0	; where to read shift keys
retikbd:	dc.l	0	; return address from patch
o_tail:	dc.w	0	; old tail index
v_x1:	dc.w	0	; VRECT elements of rect to clear
v_x2:	dc.w	0
sr_0:	dc.w	0	; status register storage
reg_0:	dc.l	0,0,0,0,0,0,0	; d0-d6
	dc.l	0,0,0,0		; a0-a3



; Here's the non-resident part ...

tsr_end:
	pea	patch_ikbd(pc)
	move	#38,-(sp)	; Supexec
	trap	#14	; Xbios
	addq	#6,sp
	lea	ok_msg(pc),a0
	bsr	print
	move.l	#tsr_end,d0
	sub.l	#tsr_start,d0
	add.l	#256,d0	; amount of memory to keep
	clr	-(sp)
	move.l	d0,-(sp)
	move	#$31,-(sp)	; Ptermres
	trap	#1	; Gemdos, no return
	
patch_ikbd:			; Supexec me ...
  ; get a-line pointer
  	.dc.w	$a000
  	move.l	a0,alinep
  ; set up pointers to key buffer and tail index
	move	#1,-(sp)	; device ID: keyboard
	move	#$e,-(sp)	; Iorec
	trap	#14	; Xbios
	addq	#4,sp
	move.l	d0,a0
	add.l	#8,d0
	move.l	(a0),bufp
	move.l	d0,tailp

	move	#$22,-(sp)	; Kbdvbase
	trap	#14
	addq	#2,sp
	add.l	#32,d0	; offset in vector table
	move.l	d0,a0	; a0 points to ikbd interrupt
	move.l	(a0),ADJ1+2	; get old vector
pi_loop:	tst.b	4(a0)	; is packet being sent?
	bne	pi_loop	;  yes: wait
	move	sr,d1	; save status reg
	or	#$700,sr	; go to IPL 7
	tst.b	4(a0)	; check again--packet being sent?
	beq	pi_skip	;  no: okay
	move	d1,sr	;  yes: let the packet through
	bra	pi_loop      ;   and loop back
pi_skip:	move.l	#ikbd1,(a0)	; do the actual patch
	move.l	$4f2,a0	; _sysbase
	move	2(a0),d0	; TOS version
	cmp	#$0100,d0
	bne	not_100
	move.l	#$e1b,shiftp
	bra	pi_out
not_100:	move.l	$24(a0),shiftp
pi_out:	move	d1,sr	; restore IPL
	rts
print:	move.l	a0,-(sp)
	move	#9,-(sp)
	trap	#1
	addq	#6,sp
	rts

ok_msg:	.dc.b	13,10,' Hotkey for LED-Panel v3.0 resident',13,10,0
